@@ -1,7 +1,7 @@
 /**
  * MIT License
  * 
- * Copyright (c) 2018 Tessil
+ * Copyright (c) 2018 Thibaut Goetghebuer-Planchon <tessil@gmx.com>
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -77,7 +77,7 @@ using test_types = boost::mpl::list<
                         tsl::hopscotch_map<std::string, std::string, mod_hash<9>, std::equal_to<std::string>, 
                             std::allocator<std::pair<std::string, std::string>>, 62, false, tsl::hh::mod_growth_policy<>>,
                         tsl::hopscotch_map<std::string, std::string, mod_hash<9>, std::equal_to<std::string>, 
-                            std::allocator<std::pair<std::string, std::string>>, 62, false, tsl::hh::mod_growth_policy<std::ratio<4, 3>>>
+                            std::allocator<std::pair<std::string, std::string>>, 30, true, tsl::hh::mod_growth_policy<std::ratio<4, 3>>>
                         >;
                                     
                               
@@ -122,14 +122,14 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(test_insert, HMap, test_types) {
 }
 
 
-// Get nothrow_move_construbtible elements into the overflow list before rehash.
+// Get nothrow_move_constructible elements into the overflow list before rehash.
 static const unsigned int overflow_mod = 50;
 using test_overflow_rehash_types = boost::mpl::list<
                     tsl::hopscotch_map<std::int64_t, move_only_test, mod_hash<overflow_mod>, std::equal_to<std::int64_t>, 
                             std::allocator<std::pair<std::int64_t, move_only_test>>, 6>,
                     tsl::bhopscotch_map<std::int64_t, move_only_test, mod_hash<overflow_mod>, std::equal_to<std::int64_t>, 
                             std::less<std::int64_t>, std::allocator<std::pair<const std::int64_t, move_only_test>>, 6>>;                   
-BOOST_AUTO_TEST_CASE_TEMPLATE(test_insert_overflow_rehash_nothrow_move_construbtible, HMap, test_overflow_rehash_types) {    
+BOOST_AUTO_TEST_CASE_TEMPLATE(test_insert_overflow_rehash_nothrow_move_constructible, HMap, test_overflow_rehash_types) {    
     // insert x/mod values, insert x values, check values
     static_assert(std::is_nothrow_move_constructible<typename HMap::value_type>::value, "");
     
@@ -172,7 +172,7 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(test_insert_overflow_rehash_nothrow_move_construbt
     }
 }
 
-// Get !nothrow_move_construbtible elements into the overflow list before rehash.
+// Get !nothrow_move_constructible elements into the overflow list before rehash.
 using test_overflow_rehash_copy_only_types = boost::mpl::list<
                     tsl::hopscotch_map<std::int64_t, copy_only_test, mod_hash<overflow_mod>, std::equal_to<std::int64_t>, 
                             std::allocator<std::pair<std::int64_t, copy_only_test>>, 6>,
@@ -721,31 +721,31 @@ BOOST_AUTO_TEST_CASE(test_modify_value_through_iterator) {
  * constructor
  */
 BOOST_AUTO_TEST_CASE(test_extreme_bucket_count_value_construction) {
-    BOOST_CHECK_THROW((tsl::hopscotch_map<int, int, std::hash<int>, std::equal_to<int>, 
+    TSL_HH_CHECK_THROW((tsl::hopscotch_map<int, int, std::hash<int>, std::equal_to<int>, 
                                          std::allocator<std::pair<int, int>>, 62, false, 
                                          tsl::hh::power_of_two_growth_policy<2>>
                             (std::numeric_limits<std::size_t>::max())), std::length_error);
     
-    BOOST_CHECK_THROW((tsl::hopscotch_map<int, int, std::hash<int>, std::equal_to<int>, 
+    TSL_HH_CHECK_THROW((tsl::hopscotch_map<int, int, std::hash<int>, std::equal_to<int>, 
                                          std::allocator<std::pair<int, int>>, 62, false, 
                                          tsl::hh::power_of_two_growth_policy<2>>
                             (std::numeric_limits<std::size_t>::max()/2 + 1)), std::length_error);
     
     
     
-    BOOST_CHECK_THROW((tsl::hopscotch_map<int, int, std::hash<int>, std::equal_to<int>, 
+    TSL_HH_CHECK_THROW((tsl::hopscotch_map<int, int, std::hash<int>, std::equal_to<int>, 
                                          std::allocator<std::pair<int, int>>, 62, false, 
                                          tsl::hh::prime_growth_policy>
                             (std::numeric_limits<std::size_t>::max())), std::length_error);
     
-    BOOST_CHECK_THROW((tsl::hopscotch_map<int, int, std::hash<int>, std::equal_to<int>, 
+    TSL_HH_CHECK_THROW((tsl::hopscotch_map<int, int, std::hash<int>, std::equal_to<int>, 
                                          std::allocator<std::pair<int, int>>, 62, false, 
                                          tsl::hh::prime_growth_policy>
                             (std::numeric_limits<std::size_t>::max()/2)), std::length_error);
     
     
     
-    BOOST_CHECK_THROW((tsl::hopscotch_map<int, int, std::hash<int>, std::equal_to<int>, 
+    TSL_HH_CHECK_THROW((tsl::hopscotch_map<int, int, std::hash<int>, std::equal_to<int>, 
                                          std::allocator<std::pair<int, int>>, 62, false, 
                                          tsl::hh::mod_growth_policy<>>
                             (std::numeric_limits<std::size_t>::max())), std::length_error);
@@ -945,6 +945,29 @@ BOOST_AUTO_TEST_CASE(test_copy_constructor_and_operator) {
     BOOST_CHECK(map_copy == map_copy3);
 }
 
+BOOST_AUTO_TEST_CASE(test_copy_constructor_empty) {
+    tsl::hopscotch_map<std::string, int> map(0);
+    tsl::hopscotch_map<std::string, int> map_copy(map);
+    
+    BOOST_CHECK(map.empty());
+    BOOST_CHECK(map_copy.empty());
+    
+    BOOST_CHECK(map.find("") == map.end());
+    BOOST_CHECK(map_copy.find("") == map_copy.end());
+}
+
+BOOST_AUTO_TEST_CASE(test_copy_operator_empty) {
+    tsl::hopscotch_map<std::string, int> map(0);
+    tsl::hopscotch_map<std::string, int> map_copy(16);
+    map_copy = map;
+    
+    BOOST_CHECK(map.empty());
+    BOOST_CHECK(map_copy.empty());
+    
+    BOOST_CHECK(map.find("") == map.end());
+    BOOST_CHECK(map_copy.find("") == map_copy.end());
+}
+
 /**
  * at
  */
@@ -954,7 +977,18 @@ BOOST_AUTO_TEST_CASE(test_at) {
     
     BOOST_CHECK_EQUAL(map.at(0), 10);
     BOOST_CHECK_EQUAL(map.at(-2), 20);
-    BOOST_CHECK_THROW(map.at(1), std::out_of_range);
+    TSL_HH_CHECK_THROW(map.at(1), std::out_of_range);
+}
+
+/**
+ * contains
+ */
+BOOST_AUTO_TEST_CASE(test_contains) {
+    tsl::hopscotch_map<std::int64_t, std::int64_t> map = {{0, 10}, {-2, 20}};
+    
+    BOOST_CHECK(map.contains(0));
+    BOOST_CHECK(map.contains(-2));
+    BOOST_CHECK(!map.contains(-3));
 }
 
 /**
@@ -976,9 +1010,11 @@ BOOST_AUTO_TEST_CASE(test_equal_range) {
 /**
  * operator[]
  */
-BOOST_AUTO_TEST_CASE(test_access_operator) {
+using test_access_operator_types = boost::mpl::list<tsl::hopscotch_map<std::int64_t, std::int64_t>, 
+                                                    tsl::bhopscotch_map<std::int64_t, std::int64_t>>;
+BOOST_AUTO_TEST_CASE_TEMPLATE(test_access_operator, HMap, test_access_operator_types) {
     // insert x values, use at for known and unknown values.
-    tsl::hopscotch_map<std::int64_t, std::int64_t> map = {{0, 10}, {-2, 20}};
+    HMap map = {{0, 10}, {-2, 20}};
     
     BOOST_CHECK_EQUAL(map[0], 10);
     BOOST_CHECK_EQUAL(map[-2], 20);
@@ -1146,7 +1182,7 @@ BOOST_AUTO_TEST_CASE(test_heterogeneous_lookups) {
     
     BOOST_CHECK_EQUAL(map.at(addr1), 4);
     BOOST_CHECK_EQUAL(map.at(addr2), 5);
-    BOOST_CHECK_THROW(map.at(addr_unknown), std::out_of_range);
+    TSL_HH_CHECK_THROW(map.at(addr_unknown), std::out_of_range);
     
     
     BOOST_REQUIRE(map.find(addr1) != map.end());
@@ -1194,8 +1230,11 @@ BOOST_AUTO_TEST_CASE(test_empty_map) {
     BOOST_CHECK_EQUAL(map.count(""), 0);
     BOOST_CHECK_EQUAL(map.count("test"), 0);
     
-    BOOST_CHECK_THROW(map.at(""), std::out_of_range);
-    BOOST_CHECK_THROW(map.at("test"), std::out_of_range);
+    BOOST_CHECK(!map.contains(""));
+    BOOST_CHECK(!map.contains("test"));
+    
+    TSL_HH_CHECK_THROW(map.at(""), std::out_of_range);
+    TSL_HH_CHECK_THROW(map.at("test"), std::out_of_range);
     
     auto range = map.equal_range("test");
     BOOST_CHECK(range.first == range.second);
@@ -1232,7 +1271,16 @@ BOOST_AUTO_TEST_CASE(test_precalculated_hash) {
     BOOST_CHECK_EQUAL(map_const.at(3, map_const.hash_function()(3)), -3);
     
     BOOST_REQUIRE_NE(map.hash_function()(2), map.hash_function()(3));
-    BOOST_CHECK_THROW(map.at(3, map.hash_function()(2)), std::out_of_range);
+    TSL_HH_CHECK_THROW(map.at(3, map.hash_function()(2)), std::out_of_range);
+    
+    /**
+     * count
+     */
+    BOOST_CHECK(map.contains(3, map.hash_function()(3)));
+    BOOST_CHECK(map_const.contains(3, map_const.hash_function()(3)));
+    
+    BOOST_REQUIRE_NE(map.hash_function()(2), map.hash_function()(3));
+    BOOST_CHECK(!map.contains(3, map.hash_function()(2)));
     
     /**
      * count
